using System;
using System.Reflection;
using System.Threading.Tasks;
using _Combo;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using UnityEngine;

namespace _Core
{
    internal static class TapTapLogin
    {
        public static Type tapLoginType = Assembly.Load("TapSDK.Login.Runtime").GetType("TapSDK.Login.TapTapLogin");
        public static Type tapTapInitType = Assembly.Load("TapSDK.Core.Runtime").GetType("TapSDK.Core.TapTapSDK");
        public static Type tapTapSdkOptionsType = Assembly.Load("TapSDK.Core.Runtime").GetType("TapSDK.Core.TapTapSdkOptions");
        public static Type tapTapSdkBaseOptionsType = Assembly.Load("TapSDK.Core.Runtime").GetType("TapSDK.Core.TapTapSdkBaseOptions");
        public static Type tapTapComplianceOptionType = Assembly.Load("TapSDK.Compliance").GetType("TapSDK.Compliance.TapTapComplianceOption");

        public static void Init(string clientId, string clientToken, bool showSwitchAccount, bool useAgeRange)
        {
            if (tapTapInitType == null)
            {
                Log.W("Failed to get type TapTapSDK");
            }
            else
            {
                // 获取反射的 Init 方法
                MethodInfo methodWithTwoParams = tapTapInitType.GetMethod(
                    "Init",
                    BindingFlags.Static | BindingFlags.Public,
                    null,
                    new Type[] { tapTapSdkOptionsType, tapTapSdkBaseOptionsType.MakeArrayType() }, 
                    null);

                if (methodWithTwoParams != null)
                {
                    try
                    {
                        // 创建并初始化 TapTapSdkOptions 的实例
                        object tapTapSdkOptionsInstance = Activator.CreateInstance(tapTapSdkOptionsType);

                        // 创建 TapTapSdkBaseOptions[] 数组实例
                        Array tapTapBaseOptionsArrayInstance = Array.CreateInstance(tapTapSdkBaseOptionsType, 1);

                        // 创建 TapTapComplianceOption 的实例并放入数组中
                        object tapTapComplianceOptionInstance = Activator.CreateInstance(tapTapComplianceOptionType);
                        tapTapBaseOptionsArrayInstance.SetValue(tapTapComplianceOptionInstance, 0);

                        if (tapTapSdkOptionsInstance != null && tapTapBaseOptionsArrayInstance != null)
                        {
                            // 设置 TapTapSdkOptions 的属性
                            FieldInfo clientIdProperty = tapTapSdkOptionsType.GetField("clientId");
                            FieldInfo clientTokenProperty = tapTapSdkOptionsType.GetField("clientToken");

                            if (clientIdProperty != null && clientTokenProperty != null)
                            {
                                clientIdProperty.SetValue(tapTapSdkOptionsInstance, clientId);
                                clientTokenProperty.SetValue(tapTapSdkOptionsInstance, clientToken);
                            }
                            else
                            {
                                Console.WriteLine("Failed to get properties of TapTapSdkOptions.");
                                return;
                            }

                            // 设置 TapTapComplianceOption 的属性
                            FieldInfo showSwitchAccountProperty = tapTapComplianceOptionType.GetField("showSwitchAccount");
                            FieldInfo useAgeRangeProperty = tapTapComplianceOptionType.GetField("useAgeRange");

                            if (showSwitchAccountProperty != null && useAgeRangeProperty != null)
                            {
                                showSwitchAccountProperty.SetValue(tapTapComplianceOptionInstance, showSwitchAccount);
                                useAgeRangeProperty.SetValue(tapTapComplianceOptionInstance, useAgeRange);
                            }
                            else
                            {
                                Console.WriteLine("Failed to get properties of TapTapComplianceOption.");
                                return;
                            }

                            // 调用 Init 方法
                            methodWithTwoParams.Invoke(null, new object[] { tapTapSdkOptionsInstance, tapTapBaseOptionsArrayInstance });
                            Console.WriteLine("Method Init invoked successfully.");
                        }
                    }
                    catch(Exception e)
                    {
                        Console.WriteLine(e);
                    }
                }
                else
                {
                    Console.WriteLine("Failed to find Init method.");
                }
            }
        }

        public static async Task LoginWithScopes(string[] scopes, Action<TapTapLoginResult> onSuccess, Action<InternalError> onError)
        {
            if(tapLoginType == null)
            {
                Log.W("Failed to get type TapLogin");
            }
            else
            {
                MethodInfo methodWithParam = tapLoginType.GetMethod
                (
                    "LoginWithScopes",
                    BindingFlags.Instance | BindingFlags.Public,
                    null,
                    new Type[] { typeof(string[]) },
                    null
                );

                if (methodWithParam != null)
                {
                    try
                    {
                        ConstructorInfo constructor = tapLoginType.GetConstructor(
                            BindingFlags.Instance | BindingFlags.NonPublic, // 绑定标志，指定查找私有非静态构造函数
                            null,                                            // 无重载绑定约束
                            Type.EmptyTypes,                                 // 参数为空，表示无参构造函数
                            null
                        );

                        if (constructor == null)
                        {
                            Log.W("Failed to get private constructor for TapLogin");
                            onError(new InternalError("Failed to get private constructor for TapLogin", new LoginError("taptap")));
                            return;
                        }

                        // 使用私有构造函数创建实例
                        object targetInstance = constructor.Invoke(null);

                        Task result = (Task)methodWithParam.Invoke(targetInstance, new object[] { scopes });
                        await result;

                        PropertyInfo resultProperty = result.GetType().GetProperty("Result");
                        object accessToken = resultProperty.GetValue(result);
                        MethodInfo toJsonMethod = accessToken.GetType().GetMethod("ToJson");
                        string json = (string)toJsonMethod.Invoke(accessToken, null);
                        
                        // 用动态转换器将 JSON 转换为 intermediate 对象
                        JObject temporaryObject = JObject.Parse(json);

                        if (temporaryObject["access_token"]?["scope"] != null)
                        {
                            // 手动解析 scope 字段为集合形式
                            string scopeString = temporaryObject["access_token"]["scope"].ToString();
                            string[] scopeArray = scopeString.Split(' '); // 按空格拆分字符串
                            temporaryObject["access_token"]["scope"] = JArray.FromObject(scopeArray); // 替换字段内容为数组
                        }

                        // 最终反序列化为 TapTapLoginResult
                        string updatedJson = temporaryObject.ToString();
                        var loginResult = JsonConvert.DeserializeObject<TapTapLoginResult>(updatedJson);

                        Log.I(loginResult);
                        // 正常返回结果
                        onSuccess?.Invoke(loginResult);
                    }
                    catch (TaskCanceledException)
                    {
                        onError(new InternalError(new UserCancelled()));
                    }
                    catch (Exception e)
                    {
                        Log.E(e);
                        onError(new InternalError(e.Message, new LoginError("taptap")));
                    }
                }
                else
                {
                    Log.W("Failed to get method Init");
                    onError(new InternalError("Failed to get method Init", new LoginError("taptap")));
                }
            }
        }

        public static void Logout()
        {
            if(tapLoginType == null)
            {
                Log.W("Failed to get type TapLogin");
            }
            else
            {
                MethodInfo logoutMethod = tapLoginType.GetMethod("Logout", BindingFlags.Instance | BindingFlags.Public);

                if (logoutMethod != null)
                {
                    ConstructorInfo constructor = tapLoginType.GetConstructor(
                            BindingFlags.Instance | BindingFlags.NonPublic, // 绑定标志，指定查找私有非静态构造函数
                            null,                                            // 无重载绑定约束
                            Type.EmptyTypes,                                 // 参数为空，表示无参构造函数
                            null
                        );

                    if (constructor == null)
                    {
                        Log.W("Failed to get private constructor for Logout");
                        return;
                    }

                    // 使用私有构造函数创建实例
                    object targetInstance = constructor.Invoke(null);

                    logoutMethod.Invoke(targetInstance, null);
                    Log.I("Logout method invoked successfully.");
                }
                else
                {
                    Log.E("Logout method not found.");
                }
            }
        }

        public async static Task GetCurrentTapAccount(Action<TapTapLoginResult> onSuccess, Action<InternalError> onError)
        {
            if(tapLoginType == null)
            {
                Log.W("Failed to get type TapLogin");
            }
            else
            {
                try
                {
                    MethodInfo getAccessTokenMethod = tapLoginType.GetMethod("GetCurrentTapAccount", BindingFlags.Instance | BindingFlags.Public);

                    if (getAccessTokenMethod != null)
                    {
                        ConstructorInfo constructor = tapLoginType.GetConstructor(
                            BindingFlags.Instance | BindingFlags.NonPublic, // 绑定标志，指定查找私有非静态构造函数
                            null,                                            // 无重载绑定约束
                            Type.EmptyTypes,                                 // 参数为空，表示无参构造函数
                            null
                        );

                        if (constructor == null)
                        {
                            Log.W("Failed to get private constructor for TapLogin");
                            onError(new InternalError("Failed to get private constructor for TapLogin", new LoginError("taptap")));
                            return;
                        }

                        // 使用私有构造函数创建实例
                        object targetInstance = constructor.Invoke(null);

                        Task result = (Task)getAccessTokenMethod.Invoke(targetInstance, null);
                        await result;

                       PropertyInfo resultProperty = result.GetType().GetProperty("Result");
                        object accessToken = resultProperty.GetValue(result);
                        MethodInfo toJsonMethod = accessToken.GetType().GetMethod("ToJson");
                        string json = (string)toJsonMethod.Invoke(accessToken, null);
                        
                        // 用动态转换器将 JSON 转换为 intermediate 对象
                        JObject temporaryObject = JObject.Parse(json);

                        if (temporaryObject["access_token"]?["scope"] != null)
                        {
                            // 手动解析 scope 字段为集合形式
                            string scopeString = temporaryObject["access_token"]["scope"].ToString();
                            string[] scopeArray = scopeString.Split(' '); // 按空格拆分字符串
                            temporaryObject["access_token"]["scope"] = JArray.FromObject(scopeArray); // 替换字段内容为数组
                        }

                        // 最终反序列化为 TapTapLoginResult
                        string updatedJson = temporaryObject.ToString();
                        var loginResult = JsonConvert.DeserializeObject<TapTapLoginResult>(updatedJson);

                        Log.I(loginResult);
                        // 正常返回结果
                        onSuccess?.Invoke(loginResult);
                    }
                    else
                    {
                        Log.E("GetCurrentTapAccount method not found.");
                        onError(new InternalError("GetCurrentTapAccount method not found.", new LoginError("taptap")));
                    }
                }
                catch (Exception e)
                {
                    onError(new InternalError(e.Message, new LoginError("taptap")));
                }
            }
        }
    }
}