using System;
using System.Collections.Generic;
using _Combo;
using _Combo.Events;
using _Core;
using _Core.Storage;

namespace Seayoo.ComboSDK.Windows.TapTap
{
    public class TapTapManager : Singleton<TapTapManager>
    {
        private TapTapManager() { }
        private static TapTapManager _instance;
        private const string autoLoginKey = "combosdk_taptap_auto_login";
        private TapTapLoginResult loginResult;

        public static TapTapManager GetInstance()
        {

            if (_instance == null)
            {
                if (_instance == null)
                {
                    _instance = new TapTapManager();
                }
            }
            return _instance;
        }

        public void Init()
        {
            TapTapLogin.Init(Config.clientId, Config.clientToken, true, true);
            Action<int, string> callback = (code, extra) =>
            {
                switch (code)
                {
                    case 500:
                        if(string.IsNullOrEmpty(loginResult.openId))
                        {
                            var error = new LoginError("taptap", LoginErrorType.ACCOUNT_INVALID, thirdPartyMessage: "profile.openid is not available");
                            LoginFailedEvent.Invoke(new LoginFailedEvent {error = new InternalError(I18n.T("credential_expired_error"), error)});
                            CloseLoginViewEvent.Invoke();
                            break;
                        }
                        var credential = new TaptapCredential()
                        {
                            kid = loginResult.accessToken.kid,
                            macKey = loginResult.accessToken.macKey,
                            ageRange = TapTapCompliance.GetAgeRange(),
                            name = loginResult.name
                        };
                        if(GlobalParameters.Instance.TaptapMustLinkSeayooAccount)
                        {
                            CheckLinkIdentityEvent.Invoke(new CheckLinkIdentityEvent {
                                externalId = loginResult.openId,
                                credential = credential
                            });
                        }
                        else
                        {
                            LoginSuccessEvent.Invoke(new LoginSuccessEvent {
                                credential = credential
                            });
                            CloseLoginViewEvent.Invoke();
                        }
                        break;
                    case 1030:
                    case 1050:
                        break;
                    case 1001:
                    case 1000:
                        LoginSwitchAccountEvent.Invoke(new LoginSwitchAccountEvent());
                        break;
                    case 9002:
                        var loginError = new LoginError("taptap", LoginErrorType.REAL_NAME_NOT_SUPPORT, 9002, I18n.T("real_name_cancelled_error"));
                        LoginFailedEvent.Invoke(new LoginFailedEvent {error = new InternalError(I18n.T("real_name_required_error"), loginError)});
                        CloseLoginViewEvent.Invoke();
                        break;
                    case 1100:
                        UIController.Instance.ShowAlertView(
                            I18n.T("common_dialog_title"),
                            I18n.T("fangchenmi_taptap_age_dialog_text"),
                            I18n.T("acknowledge_button"),
                            () => {
                                Logout();
                            }
                        );
                        break;
                    case 1200:
                        var err = new LoginError("taptap", LoginErrorType.UNKNOWN_ERROR, 1200, cause: new NetworkError());
                        LoginFailedEvent.Invoke(new LoginFailedEvent {error = new InternalError(I18n.T("network_error"), err)});
                        CloseLoginViewEvent.Invoke();
                        break;
                    default:
                        LoginFailedEvent.Invoke(new LoginFailedEvent {
                            error = new InternalError(extra)
                        });
                        CloseLoginViewEvent.Invoke();
                        break;
                }
            };
            TapTapCompliance.RegisterComplianceCallback(callback);
        }

        public async void Login()
        {
            List<string> scopes = new List<string>
            {
                "public_profile"
            };
            await TapTapLogin.LoginWithScopes(scopes.ToArray(), result =>
            {
                loginResult = result;
                TapTapCompliance.Startup(result.unionId);
            }, (err)=>{
                var userCancelled = ErrorExtensions.Convert<UserCancelled>(err);
                if(userCancelled != null)
                {
                    LoginCancelEvent.Invoke(new LoginCancelEvent {});
                    CloseLoginViewEvent.Invoke();
                }
                else
                {
                    LoginFailedEvent.Invoke(new LoginFailedEvent {
                        error = err
                    });
                    CloseLoginViewEvent.Invoke();
                }
            });
        }

        public void Logout()
        {
            TapTapLogin.Logout();
            TapTapCompliance.Exit();
        }

        public async void AutoLogin()
        {
            await TapTapLogin.GetCurrentTapAccount(result =>
            {
                loginResult = result;
                TapTapCompliance.Startup(result.unionId);
            }, (err)=>{
                LoginFailedEvent.Invoke(new LoginFailedEvent {
                    error = err
                });
                CloseLoginViewEvent.Invoke();
            });
        }

        public void SetAutoLoginEnabled(bool enabled)
        {
            UserPrefs.SetBool(autoLoginKey, enabled);
        }
        public bool GetAutoLoginEnabled()
        {
            return UserPrefs.GetBool(autoLoginKey);
        }
    }
}